# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

import logging
import os
import inspect
from inspect import getframeinfo, stack
import logging.handlers
from os import listdir
from os.path import isfile, join
import datetime
import six
from six.moves import configparser
CONF = configparser.ConfigParser()
configPath = '/opt/ibm/novalinkAutoInstaller/conf/config.cfg'
CONF.read(configPath)


class AutoLogger:
    def __init__(self, module_name, count=0):
        self.logger_module = logging.getLogger(module_name)
        self.logger_module.setLevel(logging.INFO)
        if CONF.get('logging', 'LOG_DEBUG'):
            self.logger_module.setLevel(logging.DEBUG)
        logfilePath = CONF.get('logging', 'LOG_FILE_LOCATION')
        if not os.path.exists(logfilePath):
            os.makedirs(logfilePath)
        onlyfilesList = [fileList for fileList in listdir(logfilePath)
                         if isfile(join(logfilePath, fileList))]
        currentFileName = "nl_"+str(datetime.datetime.now().strftime(
                          "%Y%m%d_%H_%M_%S"))+str(".log")
        if len(onlyfilesList) > 0:
            if count == 0:
                currentFileName = onlyfilesList[-1]
        if len(onlyfilesList) > int(CONF.get(
                                    'logging', 'TOTAL_BACKUP_FILE_COUNT')):
            os.remove(logfilePath+"/"+str(onlyfilesList[0]))
        logPathWithFileName = os.path.abspath(
            logfilePath+'/'+str(currentFileName))
        fileSize = 0
        if len(onlyfilesList) > 0:
            if count == 0:
                try:
                    fileSize = os.path.getsize(logPathWithFileName)
                except Exception as e:
                    print(e)

        confFileSize = int(CONF.get('logging', 'MAX_FILE_SIZE_IN_MB'))
        # Getting fileSize in bytes from system
        # Converting file size from MB to Byte
        confFileSize = confFileSize * 1000 * 1000
        if fileSize > confFileSize:
            currentFileName = "novalink_"+str(datetime.datetime.now().strftime(
                              "%Y%m%d_%H_%M_%S"))+str(".log")
            logPathWithFileName = os.path.abspath(
                logfilePath+'/'+str(currentFileName))
        # create a file which logs and debug messages
        logfile = logging.FileHandler(logPathWithFileName)
        if CONF.get('logging', 'LOG_DEBUG'):
            logfile.setLevel(logging.DEBUG)
        console = logging.StreamHandler()
        console.setLevel(logging.INFO)
        # create formatter and add it to the log file
        fileformatter = logging.Formatter(
            '%(asctime)s - %(levelname)s - %(message)s ',
            datefmt='%m/%d/%Y %I:%M:%S %p')
        consoleformatter = logging.Formatter('%(levelname)s - %(message)s')
        logfile.setFormatter(fileformatter)
        console.setFormatter(consoleformatter)

        # add handler to the logger
        self.logger_module.addHandler(logfile)

    # add log file rotation
    def debug(self, message):
        callerInfo = getframeinfo(stack()[1][0])
        file_name = str(callerInfo.filename)
        file_name_list = str(
            file_name).split("novalinkAutoInstaller/")
        if len(file_name_list) > 1:
            file_name = file_name_list[1]
        self.logger_module.debug(str(file_name)+" - "+str(
            callerInfo.function)+" - "+str(callerInfo.lineno)+" - "+message)

    def info(self, message):
        callerInfo = getframeinfo(stack()[1][0])
        file_name = str(callerInfo.filename)
        file_name_list = str(
            file_name).split("novalinkAutoInstaller/")
        if len(file_name_list) > 1:
            file_name = file_name_list[1]
        self.logger_module.info(str(file_name)+" - "+str(
            callerInfo.function)+" - "+str(callerInfo.lineno)+" - "+message)

    def error(self, message):
        callerInfo = getframeinfo(stack()[1][0])
        file_name = str(callerInfo.filename)
        file_name_list = str(
            file_name).split("novalinkAutoInstaller/")
        if len(file_name_list) > 1:
            file_name = file_name_list[1]
        self.logger_module.error(str(file_name)+" - "+str(
            callerInfo.function)+" - "+str(callerInfo.lineno)+" - "+message)

    def warning(self, message):
        callerInfo = getframeinfo(stack()[1][0])
        file_name = str(callerInfo.filename)
        file_name_list = str(
            file_name).split("novalinkAutoInstaller/")
        if len(file_name_list) > 1:
            file_name = file_name_list[1]
        self.logger_module.warn(str(file_name)+" - "+str(
            callerInfo.function)+" - "+str(callerInfo.lineno)+" - "+message)
