# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

import paramiko
import os
from novalinkAutoInstaller.src.Common import autoLogger

log = autoLogger.AutoLogger("SSHConnectionModule", 1)


class SSHConnection:

    def createSSHconnection(self, ipAddress, port, username, password):
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            client.connect(
                str(ipAddress), int(port), str(username), str(password))
            return client
        except IOError as error:
            # log.error(str(error))
            return {"error": error, "error_message": "IOError", "code": 201}
        except paramiko.PasswordRequiredException as error:
            log.error(str(error))
            return {"error": error,
                    "error_message": "PasswordRequiredException", "code": 201}
        except paramiko.AuthenticationException as error:
            log.error(str(error))
            return {"error": error,
                    "error_message": "AuthenticationException",
                    "code": 201}
        except Exception as e:
            log.error(str(e))
            return {"error": e,
                    "error_message": "SSHConnectionException",
                    "code": 201}

    def executeCommand(self, client, command):
        try:
            stdin, stdout, stderr = client.exec_command(command)
            exit_status = stdout.channel.recv_exit_status()
            return {"code": exit_status,
                    "stdout": stdout,
                    "stderr": stderr}
        except Exception as e:
            log.error(str(e))
            return {"code": 3,
                    "stdout": str(e),
                    "stderr": str(e),
                    "exception": str(e)}

    def executeViosInstallation(self, client, command):
        try:
            log.info("Inside executeViosInstallation function")
            stdin, stdout, stderr = client.exec_command(command)
            for installInfo in stdout:
                print(installInfo)
                if 'Finished' in installInfo:
                    print("VIOS install is starting, Installation \
                        process can be seen on HMC console.")
                    return {"code": 0,
                            "stdout": stdout,
                            "stderr": stderr}
        except Exception as e:
            log.error("An error occurred in executeViosInstallation: "+str(e))
            return {"code": 3,
                    "stdout": str(e),
                    "stderr": str(e),
                    "exception": str(e)}

    def closeSSHconnection(self, client):
        try:
            client.close()
            log.info("SSH Connection closed")
        except Exception as e:
            log.error("An error occurred in SSH Connected close: "+str(e))
