# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from novalinkAutoInstaller.src.EndPoint import HMC_Handler
import os
import time
import six
from six.moves import configparser
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("NovalinkInstallerModule")


class NovalinkInstaller:

    def __init__(self, hmcObj):
        self.hmcObj = hmcObj

    def prepareNovalinkInstaller(self, info, viosList):
        log.info("Inside prepareNovalinkInstaller method")
        lparId = str(CONF.get('novalink_profile', 'LPAR_ID'))

        # virtual_eth_adapters paramters and explaination
        # slot_num : Ethernet virtual slot number
        # ieee_virtual_eth : is IEEE (0 for No, 1 for Yes)
        # port_vlan_id : PORT VLAN ID
        # addl_vlan_ids : Additional VLAN ID
        # trunk_priority: 0 -The adapter is not a trunk adapter(default)
        # 1-15 :This adapter is a trunk adapter with specified priority
        # is_required : 0 - No, 1 - Yes
        # it uses default virt switch and automatically
        # generated MAC address
        # and it does not use QoS by default.
        # Format : virtual_eth_adapters (slot_num/ieee_virtual_eth/
        #              port_vlan_id/[addl_vlan_ids]/[trunk_priority]/
        #              is_required[/[vswitch][/[mac_addr]/
        #              [allowed_os_mac_addrs]/[qos_priority]]])

        ethId = CONF.get(
            'novalink_profile', 'LPAR_ETHERNET_ID')
        mgmt_vswitch_eth_id = CONF.get(
            'novalink_profile', 'LPAR_MGMT_VSWITCH_ETH_ID')
        mgmt_vswitch_vlan_id = CONF.get(
            'vios_profile', 'VIOS_MGMT_VSWITCH_VLAN_ID')
        mgmt_switch_name = CONF.get(
            'vios_profile', 'MGMT_VSWITCH_NAME')
        vlanId = int(info['PORT VLAN ID'])
        virtual_eth_adapters = str(ethId)+"/0/"+str(
            vlanId)+"//0/0/ETHERNET0///,"+str(
            mgmt_vswitch_eth_id)+"/1/"+str(
            mgmt_vswitch_vlan_id)+"//1/1/"+str(
            mgmt_switch_name)+"///"

        log.info("virtual_eth_adapters format : "+virtual_eth_adapters)

        # virtual_scsi_adapters paramters and explaination
        # slot_num : SCSI virtual slot number
        # Adapter Type : client/Server, in case of LPAR it is "client"
        # remote_lpar_id : VIOS ID
        # remote_lpar_name : VIOS NAME
        # remote_slot_num : SCSI slot number of VIOS
        # is_required : 0 - No, 1 - Yes
        # Format : slot_num/adapter_type/[remote_lpar_id]/
        #              [remote_lpar_name]/
        #              [remote_slot_num]/is_required

        frameName = ""
        virtual_scsi_adapters = ""
        for viosInfo in viosList:
            frameName = viosInfo['frameName']
            lparSlotNum = viosInfo['lparSlotNum']
            viosId = viosInfo['profileParameters']['lpar_id']
            viosSlotNum = viosInfo['viosSlotNum']
            if virtual_scsi_adapters != "":
                virtual_scsi_adapters = virtual_scsi_adapters+','
            virtual_scsi_adapters = virtual_scsi_adapters+str(
                lparSlotNum)+"/client/"+str(viosId)+"//"+str(viosSlotNum)+"/0"
        log.info("virtual_scsi_adapters format : "+virtual_scsi_adapters)

        # Getting IO List from HMC
        # Filtering Serial Bus adapters
        # Getting DRC index of that serial bus
        # drc_index: Slot DRC index
        # slot_io_pool_id : IO Pool ID
        # is_required : 0 - No, 1 - Yes
        # Format : (drc_index/[slot_io_pool_id]/is_required)

        serialListConf = CONF.get(
            'vios_profile', 'IO_ADAPTERS_STR_SERIAL').split(",")
        io_slots = ""
        res = self.hmcObj.getIoSlots(frameName)
        for data in res:
            for serailInfo in serialListConf:
                if serailInfo in data['description']:
                    # Adding "," before adding new IO
                    # Slots in single string
                    if io_slots != "":
                        io_slots = io_slots+','
                    # Forming structure for io_slots with DRC index
                    io_slots = io_slots+str(data['drc_index'])+"//1"
        log.info("IO Slots format : "+io_slots)

        profileParameters = {}
        profileParameters['lpar_id'] = lparId

        profileParameters['lpar_env'] = CONF.get(
            'novalink_profile', 'LPAR_OS_NAME')
        profileParameters['min_mem'] = int(float(info['MIN MEMORY(GB)'])*1024)
        profileParameters['desired_mem'] = int(float(
            info['DESIRED MEMORY(GB)'])*1024)
        profileParameters['max_mem'] = int(float(info['MAX MEMORY(GB)'])*1024)

        profileParameters['proc_mode'] = CONF.get(
            'novalink_profile', 'LPAR_PROC_MODE')
        profileParameters['shared_proc_pool_id'] = CONF.get(
            'vios_profile', 'SHARED_PROC_POOL_ID')
        profileParameters['sharing_mode'] = CONF.get(
            'novalink_profile', 'LPAR_SHARING_MODE')
        profileParameters['uncap_weight'] = CONF.get(
            'novalink_profile', 'LPAR_UNCAP_WEIGHT')
        profileParameters['min_procs'] = info['MIN VIRTUAL PROCS']
        profileParameters['desired_procs'] = info['DESIRED VIRTUAL PROCS']
        profileParameters['max_procs'] = info['MAX VIRTUAL PROCS']
        profileParameters['min_proc_units'] = info['MIN SHARED PROC UNITS']
        profileParameters['desired_proc_units'] = str(
            info['DESIRED SHARED PROC UNITS'])
        profileParameters['max_proc_units'] = info['MAX SHARED PROC UNITS']
        profileParameters['all_resources'] = CONF.get(
            'novalink_profile', 'LPAR_ALL_RESOURCES')
        profileParameters['max_virtual_slots'] = CONF.get(
            'novalink_profile', 'LPAR_MAX_VIRTUAL_SLOTS')
        profileParameters['boot_mode'] = CONF.get(
           'novalink_profile', 'LPAR_BOOT_MODE')
        profileParameters['power_ctrl_lpar_ids'] = CONF.get(
            'novalink_profile', 'LPAR_POWER_CTRL_LPAR_IDS')
        profileParameters['conn_monitoring'] = CONF.get(
            'novalink_profile', 'LPAR_CONN_MONITORING')
        profileParameters['auto_start'] = CONF.get(
            'novalink_profile', 'LPAR_AUTO_START')

        profileParameters['virtual_eth_adapters'] = virtual_eth_adapters
        profileParameters['virtual_scsi_adapters'] = virtual_scsi_adapters
        if io_slots != "":
            profileParameters['io_slots'] = io_slots

        installerInfo = {}
        installerInfo['frameName'] = frameName
        installerInfo['lparName'] = info['NAME']
        installerInfo['profileName'] = CONF.get(
            'novalink_profile', 'LPAR_PROFILE_NAME')
        installerInfo['serverIp'] = info['BOOTP SERVER IP']
        installerInfo['gatewayIp'] = info['GATEWAY']
        installerInfo['subnetMask'] = info['SUBNET MASK']
        installerInfo['lparIp'] = info['IP ADDRESS']
        installerInfo['hostname'] = info['HOSTNAME(FQDN)']
        installerInfo['adapterType'] = CONF.get(
            'novalink_profile', 'LPAR_ADAPTER_TYPE')
        installerInfo['profileCreationStatus'] = False
        installerInfo['installationTriggered'] = False
        installerInfo['deleteProfile'] = False
        installerInfo['vlanId'] = vlanId
        installerInfo['diskSize'] = str(info['DISK SIZE(GB)'])+"G"
        installerInfo['diskName'] = CONF.get(
            'novalink_profile', 'LPAR_DISK_NAME')
        installerInfo['diskVG'] = CONF.get(
            'novalink_profile', 'LPAR_DISK_TO_USE')
        installerInfo['profileParameters'] = profileParameters
        return installerInfo

    # Applying profile to Novalink LPAR
    def applyProfile(self, frameName, lparName, profileName):
        log.info("Inside applyVIOSProfile method")
        applyLparProfileResponse = self.hmcObj.applyProfile(
            frameName, lparName, profileName)
        return applyLparProfileResponse

    # Creating LPAR profile for Novalink
    def createNovalinkProfile(self, info):
        log.info("Inside createNovalinkProfile method")
        frameName = info['frameName']
        lparName = info['lparName']
        profileName = info['profileName']
        profileParameters = info['profileParameters']
        createProfileResponse = self.hmcObj.createProfile(
            frameName, lparName, profileName, profileParameters)
        if createProfileResponse['code'] == 0:
            log.info("Novalink Partition and Profile created")
            applyProfileResponse = self.applyProfile(
                frameName, lparName, profileName)
            if applyProfileResponse['code'] == 0:
                log.info("Profile applied to Novalink LPAR")
                print(MSGCATALOG.NOVALINK_PROFILE_CREATE_SUCCESS.format(
                    lparName))
                return {"code": 0}
            else:
                errorRes = ""
                for err in applyProfileResponse["stdout"]:
                    errorRes = errorRes+" "+err
                print(errorRes)
                log.error(errorRes)
                return False

        else:
            errorRes = ""
            for err in createProfileResponse["stdout"]:
                errorRes = errorRes+" "+err
            print(errorRes)
            log.error(errorRes)
            return False

    # Setting HMC as PowerVM management master
    def coManagementSetMaster(self, info):
        frameName = info['frameName']
        res = self.hmcObj.coMgmtSetMaster(frameName)
        if res['code'] == 0:
            return res
        else:
            errorRes = ""
            for err in res["stdout"]:
                errorRes = errorRes+" "+err
            print(errorRes)
            log.error(errorRes)
            return res

    # Setting Novalink LPAR as PowerVM management capable
    # To enable powerVM management, value must be set to 1
    def setPowerVMCapableManagement(self, info):
        lpar_id = info['profileParameters']['lpar_id']
        frameName = info['frameName']
        res = self.hmcObj.powerVMcapableManagement(frameName, lpar_id, 1)
        if res['code'] == 0:
            return res
        else:
            errorRes = ""
            for err in res["stdout"]:
                errorRes = errorRes+" "+err
            print(errorRes)
            log.error(errorRes)
            return res

    # Main function for setting HMC as master and
    # Novalink LPAR as PowerVM management capable
    def setCoMgmtAndPowerVMCapable(self, info):
        POST_PVM_CPBL_SLEEP_TIME = CONF.get(
            'novalink_profile', 'POST_PVM_CPBL_SLEEP_TIME')
        POST_HMC_SET_MASTER_TIME = CONF.get(
            'novalink_profile', 'POST_HMC_SET_MASTER_TIME')
        res = self.coManagementSetMaster(info)
        if res['code'] == 0:
            print(MSGCATALOG.SET_CO_MGMT_MASTER)
            time.sleep(int(POST_HMC_SET_MASTER_TIME))
            res = self.setPowerVMCapableManagement(info)
            if res['code'] == 0:
                print(MSGCATALOG.SET_PVM_CPBL_MGMT)
                time.sleep(int(POST_PVM_CPBL_SLEEP_TIME))
                return True
            return False

    def lparShutdown(self, frameName, lparName):
        log.info("Inside lparShutdown method")
        res = self.hmcObj.lparChangeStateImmedShutdown(frameName, lparName)
        if res['code'] == 0:
            return res
        else:
            errorRes = ""
            for err in res["stdout"]:
                errorRes = errorRes+" "+err
            log.info(errorRes)
            log.error(errorRes)
            errorRes = ""
            for err in res["stderr"]:
                errorRes = errorRes+" "+err
            log.info(errorRes)
            log.error(errorRes)
            return res

    def activateProfile(self, info):
        log.info("Inside activateProfile method")
        lparName = info['lparName']
        frameName = info['frameName']
        res = self.hmcObj.lparActivateProfileCurrentConfiguration(
            frameName, lparName)
        if res['code'] == 0:
            log.info("Novalink VM Powering ON")
            return True
        else:
            log.error("Unable to Power on Novalink VM")
            for err in res['stdout']:
                log.error(err)
        return False

    def get_loc_code_nl(self, info):
        log.info("inside get_loc_code_nl method")
        print("Performing ping test for Novalink VM")
        res = self.hmcObj.getLocCodeNovalink(info)
        if res['code'] == 0:
            loc_code = res["stdout"].split(" ")[1].strip()
            return loc_code
        else:
            return False

    # Novalink install
    def installNovalink(self, info):
        log.info("inside installNovalink method")
        loc_code = self.get_loc_code_nl(info)
        if loc_code:
            info['loc_code'] = loc_code
            res = self.hmcObj.installNovalink(info)
            if res['code'] == 0:
                return True
        return False
