# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from novalinkAutoInstaller.src.EndPoint import HMC_Handler
import os
import time
import six
from six.moves import configparser
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("VIOSInstallerModule")


class VIOSInstaller:

    def __init__(self, hmcObj):
        self.hmcObj = hmcObj

    # Validate IP Clash using Ping Test
    def pingTest(self, ipAddress):
        log.info("Checking ping test for "+ipAddress)
        res = self.hmcObj.pingTest(ipAddress)
        return res

    # Fetching LPAR list within a managed system.
    # If any LPAR exist with a managed system, printing console log
    # and exiting from installation.
    def checkExistingLpar(self, frameName):
        log.info("Inside checkExistingLpar method")
        getLparList = self.hmcObj.listLpar(frameName)
        if len(getLparList) > 0:
            print(MSGCATALOG.LPAR_EXIST.format(frameName))
            for info in getLparList:
                print("VM name: "+info['lpar_name']+', state: '+info['state'])
                log.info("VM : "+info['lpar_name']+', state: '+info['state'])
            return True
        else:
            return False

    def getDrcIndex(self, frameName, drc_name):
        log.info("Inside getDrcIndex method")
        res = self.hmcObj.getDrcIndexUsingDrcName(frameName, drc_name)
        log.info("DRC index for "+drc_name+" is : "+str(res['drc_index']))
        return res['drc_index']

    def configureVIOS(self, frameName, viosName, login, password):
        log.info("Inside configureVIOS method")
        res = self.hmcObj.configureVIOS(frameName, viosName, login, password)
        return res

    def getRmcState(self, frameName, viosName):
        log.info("Inside getRmcState method")
        res = self.hmcObj.getViosRmcState(frameName, viosName)
        return res

    # Getting HMC network interface details
    def hmcNetworkInfo(self, info):
        log.info("Inside hmcNetworkInfo method")
        hmcIp = info['hmcIp']
        res = self.hmcObj.getNetworkInfo(hmcIp)
        networkInterface = False
        for keys, vals in res.items():
            if vals == hmcIp:
                if 'ipv4addr_' in keys:
                    networkInterface = keys.split("_")[1]
                    log.info("HMC NW Interface found as : "+networkInterface)
                    break
        return networkInterface

    def deleteVIOSProfile(self, info):
        log.info("Inside deleteVIOSProfile method")
        frameName = info['frameName']
        viosName = info['viosName']
        res = self.hmcObj.shutdownAndDeleteProfile(frameName, viosName)
        return res

    def getAdapterMac(self, info):
        log.info("Inside getAdapterMac method")
        res = self.hmcObj.lparNetbootPingTestVios(info)
        return res

    def get_sriov_log_ports(self, frameName, info):
        log.info("Inside get_sriov_log_ports method")
        ioSlots = info['sriov_io']
        portList = []
        for ioInfo in ioSlots:
            ioName = ioInfo.split('|')[0].strip()
            port_type = ioInfo.split('SRIOV-')[1].strip()
            port_type = port_type.split('Link')[0].strip()
            portInfo = self.hmcObj.get_sriov_ports_filtered(
                frameName, ioName, port_type)
            if portInfo:
                portList.append(portInfo)
        return portList

    def applyVIOSProfile(self, frameName, lparName, profileName):
        log.info("Inside applyVIOSProfile method")
        applyLparProfileResponse = self.hmcObj.applyProfile(
            frameName, lparName, profileName)
        return applyLparProfileResponse

    def set_service_partition(self, frameName, lparId):
        log.info("Inside set_service_partition method")
        res = self.hmcObj.apply_service_partition(
            frameName, lparId)
        return res

    def createVIOSProfile(self, info):
        log.info("Inside createVIOSProfile method")
        frameName = info['frameName']
        viosName = info['viosName']
        vios_id = str(info['profileParameters']['lpar_id'])
        profileName = info['profileName']
        profileParameters = info['profileParameters']
        VIOS_ID_START_INDEX = CONF.get(
            'vios_profile', 'VIOS_ID_START_INDEX')
        createProfileResponse = self.hmcObj.createProfile(
            frameName, viosName, profileName, profileParameters)
        if createProfileResponse['code'] == 0:
            log.info("VIOS Partition and Profile created")
            applyProfileResponse = self.applyVIOSProfile(
                frameName, viosName, profileName)
            if applyProfileResponse['code'] == 0:
                log.info("Profile applied to VIOS")
                if vios_id == VIOS_ID_START_INDEX:
                    res = self.set_service_partition(
                        frameName, vios_id)
                    if res['code'] == 0:
                        log.info("Service Parition setup done")
                        return {"code": 0}
                    else:
                        errorRes = ""
                        for err in res["stdout"]:
                            errorRes = errorRes+" "+err
                        log.error(errorRes)
                        return {"code": 1, "text": errorRes}
                return {"code": 0}
            else:
                errorRes = ""
                for err in applyProfileResponse["stdout"]:
                    errorRes = errorRes+" "+err
                log.error(errorRes)
                return {"code": 1, "text": errorRes}

        else:
            errorRes = ""
            for err in createProfileResponse["stdout"]:
                errorRes = errorRes+" "+err
            print(errorRes)
            log.error(errorRes)
            return {"code": 1, "text": errorRes}

    def prepareViosInstaller(self, info):
        log.info("Inside prepareViosInstaller method")
        frameName = info['HOST NAME']
        viosId = str(info['viosId'])

        # Using for loop to get DRC index of each IO Adapters
        # drc_index: Slot DRC index
        # slot_io_pool_id : IO Pool ID
        # is_required : 0 - No, 1 - Yes
        # Format : (drc_index/[slot_io_pool_id]/is_required)
        io_slots = ""
        ioSlots = info['other_io']
        for ioName in ioSlots:
            ioName = ioName.split('|')[0].strip()
            drcIndex = self.getDrcIndex(frameName, ioName)
            # Adding "," before adding new IO Slots in single string
            if io_slots != "":
                io_slots = io_slots+','
            # Forming structure for io_slots with DRC index
            io_slots = io_slots+str(drcIndex)+"//1"
        log.info("IO Slots format : "+io_slots)

        # virtual_eth_adapters paramters and explaination
        # slot_num : Ethernet virtual slot number
        # ieee_virtual_eth : is IEEE (0 for No, 1 for Yes)
        # port_vlan_id : PORT VLAN ID
        # addl_vlan_ids : Additional VLAN ID
        # trunk_priority:0 -The adapter is not a trunk adapter(default)
        # 1-15 :This adapter is a trunk adapter with specified priority
        # is_required : 0 - No, 1 - Yes
        # it uses default virt switch and automatically
        # generated MAC address
        # and it does not use QoS by default.
        # Format : virtual_eth_adapters (slot_num/ieee_virtual_eth/
        #              port_vlan_id/[addl_vlan_ids]/[trunk_priority]/
        #              is_required[/[vswitch][/[mac_addr]/
        #              [allowed_os_mac_addrs]/[qos_priority]]])

        ethId = CONF.get(
            'vios_profile', 'VIOS_ETHERNET_ID')
        mgmt_vswitch_vlan_id = CONF.get(
            'vios_profile', 'VIOS_MGMT_VSWITCH_VLAN_ID')
        mgmt_vswitch_eth_id = CONF.get(
            'vios_profile', 'VIOS_MGMT_VSWITCH_ETH_ID')
        mgmt_switch_name = CONF.get(
            'vios_profile', 'MGMT_VSWITCH_NAME')
        trunkPriority = int(info['trunkPriority'])
        vlanId = int(info['PORT VLAN ID'])
        virtual_eth_adapters = str(ethId)+"/1/"+str(
            vlanId)+"//"+str(trunkPriority)+"/1/ETHERNET0///,"+str(
            mgmt_vswitch_eth_id)+"/1/"+str(
            mgmt_vswitch_vlan_id)+"//0/0/"+str(
            mgmt_switch_name)+"///"
        log.info("virtual_eth_adapters formats : "+virtual_eth_adapters)

        # virtual_scsi_adapters paramters and explaination
        # slot_num : SCSI virtual slot number
        # Adapter Type : client/Server, in case of VIOS it is "server"
        # remote_lpar_id : Novalink LPAR ID
        # remote_lpar_name : Novalink LPAR NAME
        # remote_slot_num : SCSI slot number od Novalink LPAR
        # is_required : 0 - No, 1 - Yes
        # Format : slot_num/adapter_type/[remote_lpar_id]/
        #              [remote_lpar_name]/
        #              [remote_slot_num]/is_required
        viosSlotNum = info['viosSlotNum']
        novalinkLparId = info['novalinkLparId']
        lparSlotNum = info['lparSlotNum']
        virtual_scsi_adapters = str(viosSlotNum)+"/server/"+str(
            novalinkLparId)+"//"+str(lparSlotNum)+"/1"

        log.info("virtual_scsi_adapters format : "+virtual_scsi_adapters)

        sriov_port_list = self.get_sriov_log_ports(frameName, info)
        sriov_eth_log_ports = ""
        for sriovInfo in sriov_port_list:
            adapter_id = sriovInfo['adapter_id']
            phys_port_id = sriovInfo['phys_port_id']
            capacity = sriovInfo['min_eth_capacity_granularity']
            if sriov_eth_log_ports != "":
                sriov_eth_log_ports = sriov_eth_log_ports+','
            promisc_mode = CONF.get(
                'vios_profile', 'PROMISC_MODE')
            sriov_eth_log_ports = sriov_eth_log_ports+'\"adapter_id='+str(
                adapter_id)+':phys_port_id='+str(
                phys_port_id)+':port_vlan_id='+str(
                vlanId)+':allowed_vlan_ids=none'+':promisc_mode='+str(
                promisc_mode)+':capacity='+str(capacity)+'\"'


        log.info("sriov_eth_log_ports format : "+sriov_eth_log_ports)
        profileParameters = {}
        profileParameters['lpar_id'] = viosId

        profileParameters['lpar_env'] = CONF.get(
            'vios_profile', 'VIOS_OS_NAME')
        minMem = int(float(info['MIN MEMORY(GB)'])*1024)
        desiredMem = int(float(info['DESIRED MEMORY(GB)'])*1024)
        maxMem = int(float(info['MAX MEMORY(GB)'])*1024)

        profileParameters['min_mem'] = minMem
        profileParameters['desired_mem'] = desiredMem
        profileParameters['max_mem'] = maxMem

        profileParameters['proc_mode'] = CONF.get(
            'vios_profile', 'PROC_MODE')
        profileParameters['shared_proc_pool_id'] = CONF.get(
            'vios_profile', 'SHARED_PROC_POOL_ID')
        profileParameters['sharing_mode'] = CONF.get(
            'vios_profile', 'SHARING_MODE')
        profileParameters['uncap_weight'] = CONF.get(
            'vios_profile', 'UNCAP_WEIGHT')
        profileParameters['min_procs'] = info['MIN VIRTUAL PROCS']
        profileParameters['desired_procs'] = info['DESIRED VIRTUAL PROCS']
        profileParameters['max_procs'] = info['MAX VIRTUAL PROCS']
        profileParameters['min_proc_units'] = info['MIN SHARED PROC UNITS']
        profileParameters['desired_proc_units'] = str(
            info['DESIRED SHARED PROC UNITS'])
        profileParameters['max_proc_units'] = info['MAX SHARED PROC UNITS']
        profileParameters['all_resources'] = CONF.get(
            'vios_profile', 'ALL_RESOURCES')
        profileParameters['max_virtual_slots'] = CONF.get(
            'vios_profile', 'MAX_VIRTUAL_SLOTS')
        profileParameters['boot_mode'] = CONF.get('vios_profile', 'BOOT_MODE')
        profileParameters['power_ctrl_lpar_ids'] = CONF.get(
            'vios_profile', 'POWER_CTRL_LPAR_IDS')
        profileParameters['conn_monitoring'] = CONF.get(
            'vios_profile', 'CONN_MONITORING')
        profileParameters['io_slots'] = io_slots
        profileParameters['virtual_eth_adapters'] = virtual_eth_adapters
        profileParameters['virtual_scsi_adapters'] = virtual_scsi_adapters
        if sriov_eth_log_ports != "":
            profileParameters['sriov_eth_logical_ports'] = sriov_eth_log_ports
        profileParameters['msp'] = CONF.get(
            'vios_profile', 'MSP')
        profileParameters['auto_start'] = CONF.get(
            'vios_profile', 'AUTO_START')

        installerInfo = {}
        installerInfo['frameName'] = frameName
        installerInfo['viosName'] = info['NAME']
        installerInfo['hmcIp'] = info['hmcIp']
        installerInfo['profileName'] = CONF.get(
            'vios_profile', 'VIOS_PROFILE_NAME')
        installerInfo['serverIp'] = info['serverIp']
        installerInfo['gatewayIp'] = info['GATEWAY']
        installerInfo['subnetMask'] = info['SUBNET MASK']
        installerInfo['viosIp'] = info['IP ADDRESS']
        installerInfo['viosIsoPath'] = info['ISO PATH(HMC localpath)']
        installerInfo['adapterType'] = CONF.get(
            'vios_profile', 'VIOS_ADAPTER_TYPE')
        installerInfo['profileCreationStatus'] = False
        installerInfo['installationTriggered'] = False
        installerInfo['isRmcActive'] = False
        installerInfo['isSEAcreated'] = False
        installerInfo['deleteProfile'] = False
        installerInfo['pingTest'] = False
        installerInfo['viosSlotNum'] = viosSlotNum
        installerInfo['lparSlotNum'] = lparSlotNum
        installerInfo['vlanId'] = vlanId
        installerInfo['profileParameters'] = profileParameters
        installerInfo['sriov_io'] = info['sriov_io']
        return installerInfo

    def assign_sriov_ports(self, portInfo, frameName, viosName):
        adapterId = portInfo['adapter_id']
        pyhsPortId = portInfo['phys_port_id']
        res = self.hmcObj.assign_sriov_port(
            frameName, viosName, adapterId, pyhsPortId)
        if res['code'] == 0:
            print(MSGCATALOG.SRIOV_PORT_ASSIGNED_SUCCESS.format(
                portInfo['phys_port_loc']))
            return True
        else:
            for err in res['stdout']:
                print(err)
        return False

    def configureSRIOV(self, info):
        log.info("Inside configureSRIOV method")
        ioSlots = info['sriov_io']
        allSRIOVPortAssigned = True
        for ioName in ioSlots:
            ioName = ioName.split('|')[0].strip()
            frameName = info['frameName']
            viosName = info['viosName']
            portList = self.hmcObj.get_sriov_adapter_ports(frameName, ioName)
            log.info("SRIOV Port List : "+str(portList))
            for portInfo in portList:
                res = self.assign_sriov_ports(portInfo, frameName, viosName)
                if not res:
                    allSRIOVPortAssigned = False
        return allSRIOVPortAssigned

    def viosValidation(self, info):
        log.info("Inside viosValidation method")
        installerInfo = self.prepareViosInstaller(info)
        res = self.createVIOSProfile(installerInfo)
        if res['code'] == 0:
            print(MSGCATALOG.VIOS_PROFILE_CREATE_SUCCESS.format(
                installerInfo['viosName']))
            print(MSGCATALOG.VIOS_NETWORK_CHECK.format(
                installerInfo['viosName']))
            installerInfo['profileCreationStatus'] = True
            hmcNetworkEth = self.hmcNetworkInfo(installerInfo)
            installerInfo['hmcNetworkEth'] = hmcNetworkEth
            macRes = self.getAdapterMac(installerInfo)
            macAddress = ""
            if macRes["code"] == 0:
                macString = macRes["stdout"].split(" ")[2]
                count = 0
                installViosStart = True
                for x in range(len(macString)):
                    macAddress = macAddress+str(macString[x])
                    count = count+1
                    if count == 2:
                        macAddress = macAddress+":"
                        count = 0
                log.info("MAC Address found as : "+macAddress)
                installerInfo['macAddress'] = macAddress
                print(MSGCATALOG.VIOS_PING_SUCCESS.format(
                    installerInfo['viosName']))
                installerInfo['pingTest'] = True

            elif macRes["code"] == 1:
                for x in macRes["stdout"]:
                    print(x)
                print(MSGCATALOG.VIOS_PING_FAILED.format(
                    installerInfo['viosName']))
                installerInfo['deleteProfile'] = True
                installerInfo['pingTest'] = False
        else:
            print(MSGCATALOG.VIOS_PROFILE_CREATE_ERROR.format(
                installerInfo['viosName']))
        return installerInfo

    def installVIOS(self, info):
        log.info("Inside install method")
        installRes = self.hmcObj.installios(info)
        return installRes

    def getVirtualAdapter(self, frameName, viosName):
        log.info("Inside getVirtualAdapter method")
        res = self.hmcObj.getVirtualAdapter(frameName, viosName)
        log.info("getVirtualAdapter response : "+str(res))
        return res

    def getPortVlanID(self, frameName, viosName, vEth):
        log.info("Inside getPortVlanID method")
        res = self.hmcObj.getPortVlanID(frameName, viosName, vEth)
        log.info("getPortVlanID response : "+str(res))
        return res

    def getPhysicalAdapter(self, frameName, viosName):
        log.info("Inside getPhysicalAdapter method")
        res = self.hmcObj.getPhysicalAdapter(frameName, viosName)
        log.info("getPhysicalAdapter response : "+str(res))
        return res

    def createSEA(self, frameName, viosName, phyEth, vEth, vLAN):
        log.info("Inside createSEA method")
        res = self.hmcObj.createSEA(frameName, viosName, phyEth, vEth, vLAN)
        return res

    # SEA CREATION
    def configureSEA(self, frameName, viosName, vLAN_ID):
        log.info("Inside configureSEA method")
        # Added for loop to check virtual adapter every 15 seconds
        # Even after Ping up, sometime it takes 15 to 20 seconds
        # to get Virtual adapter status from HMC
        VIOS_GET_VIRT_ADAPTER_TIMEOUT = CONF.get(
            'vios_profile', 'VIOS_GET_VIRT_ADAPTER_TIMEOUT')
        res = []
        for x in range(int(VIOS_GET_VIRT_ADAPTER_TIMEOUT)):
            print("Checking virtual adapter information.")
            res = self.getVirtualAdapter(frameName, viosName)
            if len(res) == 0:
                log.warning("Virtual Adapter not found attempt :"+str(x))
                print("Trying again. Please wait ...")
                time.sleep(20)
            else:
                log.info("Virtual Adapter found in attempt :"+str(x))
                break

        virtAdpt = False
        for info in res:
            print("Checking Port VLAN ID.")
            vLan = self.getPortVlanID(frameName, viosName, info)
            if vLan:
                log.info("PORT VLAN ID found as :"+str(info))
                if int(vLan) == int(vLAN_ID):
                    log.info("PORT VLAN ID matched as :"+str(info))
                    virtAdpt = info

        if not virtAdpt:
            log.error("Error: Valid Virtual Adapter not found")
            print("Error: Valid Virtual Adapter not found")
            return False

        else:
            print("Checking physical adapter information.")
            phyAdpt = self.getPhysicalAdapter(frameName, viosName)
            if not phyAdpt:
                log.error("Valid Physical Adapter not found")
                print("Error: Valid Physical Adapter not found")
                return False

            if phyAdpt:
                log.error("Valid Physical Adapter found as "+str(phyAdpt))
                PA = phyAdpt[:2] + 't' + phyAdpt[2:]
                print("Creating SEA, Please wait...")
                res = self.createSEA(
                    frameName, viosName, PA, virtAdpt, vLAN_ID)
                if res['code'] == 0:
                    return True
                else:
                    for x in res['stdout']:
                        print(x)
                    return False

    # Network Creation
    def add_network_vios(self, viosInfo, ip_add):
        log.info("Inside add_network method")
        nmask = CONF.get(
            'vios_profile', 'VIOS_SUBNET_MASK')
        vlan_id = CONF.get(
            'vios_profile', 'VIOS_MGMT_VSWITCH_VLAN_ID')
        vios_name = viosInfo['viosName']
        frame_name = viosInfo['frameName']
        hostname = self.hmcObj.get_vios_hostname(frame_name, vios_name)
        if hostname:
            interface = self.hmcObj.get_empty_interface_vios(
                frame_name, vios_name, vlan_id)
            if interface:
                res = self.hmcObj.add_network_on_vios(
                    frame_name, vios_name, hostname, ip_add, nmask, interface)
                return res
        return False

    def createLV(self, info):
        res = self.hmcObj.createLV(info)
        return res

    def getvHost(self, info):
        res = self.hmcObj.getvHost(info)
        if res['code'] == 0:
            for data in res['stdout']:
                log.info("vHost Name : "+str(data.split()[0]))
                return data.split()[0]
        else:
            return False

    def mapDiskTovHost(self, info, vHost):
        res = self.hmcObj.mapDiskTovHost(info, vHost)
        return res

    def prepareVIOSforNovalink(self, viosInstallerList, NLinfo):
        isReadyCount = 0
        for info in viosInstallerList:
            info['diskSize'] = NLinfo['diskSize']
            info['diskName'] = NLinfo['diskName']
            info['diskVG'] = NLinfo['diskVG']
            time.sleep(2)
            res = self.createLV(info)
            if res['code'] == 0:
                print(MSGCATALOG.CREATE_LV_SUCCESS.format(info['viosName']))
                time.sleep(5)
                vHost = self.getvHost(info)
                if vHost:
                    res = self.mapDiskTovHost(info, vHost)
                    if res['code'] == 0:
                        isReadyCount += 1
                        print(MSGCATALOG.MAP_DISK_SUCCESS)
                    else:
                        print(MSGCATALOG.MAP_DISK_FAILED)
                else:
                    print("vHost not found")
            else:
                print(MSGCATALOG.CREATE_LV_FAILED.format(info['viosName']))
                for err in res['stdout']:
                    print(err)
        if isReadyCount == len(viosInstallerList):
            return True
        else:
            return False
