# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
from novalinkAutoInstaller.src.Validate.validator import InputValidator
from novalinkAutoInstaller.src.EndPoint import HMC_Handler
import os
import time
import six
from six.moves import configparser
CONF = configparser.ConfigParser()
CONF.read(autoLogger.configPath)
log = autoLogger.AutoLogger("validateVIOSModule")


# Comparing and validating xlsx template data from default
# configuration required for VIOS install
# Validating hardware details from xlsx template
class VIOSInputValidator(InputValidator):

    def __init__(self, hmcObj):
        self.hmcObj = hmcObj

    # Empty field validation function
    def vldEmptyFieldsVIOSname(self, viosName, VIOSHeaderName):
        log.info("Inside vldEmptyFieldsVIOSname method")
        if viosName is None or viosName == "" or viosName is False:
            print(MSGCATALOG.VIOS_NAME_EMPTY_FIELDS.format(VIOSHeaderName))
            log.info("VIOS name is required, exiting from installation")
            return 1
        else:
            return 0

    # Empty field validation function
    def vldEmptyFields(self, info):
        log.info("Inside vldEmptyFields method")
        code = 0
        for infokey, infovalue in info.items():
            res = self.emptyKeys(infokey)
            if not res:
                exit(1)
            if CONF.get('install_template', 'XLSX_DIFFERENTIATOR') == infokey:
                continue
            rescode = self.emptyValues(infovalue, infokey)
            code += rescode
        return code

    # Validate Numeric or decimal values for RAM, Processor
    def vldNumericVals(self, info):
        log.info("Inside vldNumericVals method")
        code = 0
        for infokey, infovalue in info.items():
            if infokey == "MIN MEMORY(GB)":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DESIRED MEMORY(GB)":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MAX MEMORY(GB)":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MIN SHARED PROC UNITS":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DESIRED SHARED PROC UNITS":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MAX SHARED PROC UNITS":
                resCode = self.isFloatVals(infovalue, infokey)
                code = code + resCode

            # Value of Virtual processor should
            # always be a number, not float
            elif infokey == "MIN VIRTUAL PROCS":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "DESIRED VIRTUAL PROCS":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode
            elif infokey == "MAX VIRTUAL PROCS":
                resCode = self.isDigitVals(infovalue, infokey)
                code = code + resCode
        return code

    # Mininum, desired and max memory validation
    # Desired should not be less than min memory
    # Max should not be less than desired memory
    def vldRAM(self, min, desired, max):
        log.info("Inside vldRAM method")
        VIOS_MIN_MEM = CONF.get('vios_profile', 'MIN_MEM')
        code = self.ramVld(VIOS_MIN_MEM, min, desired, max)
        return code

    # Mininum, desired and max Virtual Proc validation
    # Desired should not be less than min virtual proc
    # Max should not be less than desired virtual proc
    def vldVirtualProc(self, min, desired, max):
        log.info("Inside vldVirtualProc method")
        MIN_VIRTUAL_PROCS = CONF.get('vios_profile', 'MIN_VIRTUAL_PROCS')
        code = self.vProcVld(MIN_VIRTUAL_PROCS, min, desired, max)
        return code

    # Mininum, desired and max shared proc units validation
    # Desired should not be less than min shared proc units
    # Max should not be less than desired shared proc units
    def vldSharedProcUnits(self, min, desired, max):
        log.info("Inside vldSharedProcUnits method")
        MIN_SHARED_PROC_UNITS = CONF.get(
            'vios_profile', 'MIN_SHARED_PROC_UNITS')
        code = self.sharedProcVld(MIN_SHARED_PROC_UNITS, min, desired, max)
        return code

    def valdISOPath(self, path):
        isValidPath = self.hmcObj.checkISOlocation(path)
        if not isValidPath:
            self.valOutput('ISO PATH(HMC localpath)', 'F', str(
                path), MSGCATALOG.VIOS_ISO_PATH_INVALID)
            log.error("Invalid ISO Path, exiting installation")
            return 1
        else:
            return 0

    # PORT VLAN ID validation
    # PORT VLAN ID should always be a +ve number between 0 to 4094
    def vldVLAN_ID(self, vlanId):
        log.info("Inside vldVLAN_ID method")
        maxRange = int(CONF.get('vios_profile', 'VLAN_ID_MAX_RANGE'))
        code = self.vlanVld(vlanId, maxRange,)
        return code

    # IP Address validation main function
    def vldNetworkAttributes(self, info):
        log.info("Inside vldNetworkAttributes method")
        viosIP = info['IP ADDRESS']
        resVIOSIP = self.ipAddrVld(viosIP)
        if resVIOSIP == 0:
            resVIOSIP = self.ipAddPingVld('IP ADDRESS', viosIP)
        else:
            self.valOutput('IP ADDRESS', 'F', viosIP, str(
                MSGCATALOG.IP_ADDRESS_INVALID))

        gatewayIp = info['GATEWAY']
        resGateway = self.ipAddrVld(gatewayIp)
        if resGateway == 0:
            resGateway = self.ipAddPingVld('GATEWAY', gatewayIp, True)
        else:
            self.valOutput('GATEWAY', 'F', gatewayIp, str(
                MSGCATALOG.IP_ADDRESS_INVALID))

        subnetMask = info['SUBNET MASK']
        resSMask = self.ipAddrVld(subnetMask)
        if resSMask == 0:
            self.valOutput('SUBNET MASK', 'P', subnetMask)
        else:
            self.valOutput('SUBNET MASK', 'F', subnetMask, str(
                MSGCATALOG.IP_ADDRESS_INVALID))

        res = resVIOSIP + resGateway + resSMask
        return res

    def validateNonRequiredAdapters(self, keys, vals):
        IO_ADAPTERS_STR_SERIAL = CONF.get(
            'vios_profile', 'IO_ADAPTERS_STR_SERIAL')
        vldList = IO_ADAPTERS_STR_SERIAL.split(",")
        IO_ADAPTERS_EXPANSION_DRAWER = CONF.get(
            'vios_profile', 'IO_ADAPTERS_EXPANSION_DRAWER')
        IO_GRAPHIC_ADAPTER = CONF.get(
            'vios_profile', 'IO_GRAPHIC_ADAPTER')
        vldList.extend(IO_GRAPHIC_ADAPTER.split(","))
        vldList.extend(IO_ADAPTERS_EXPANSION_DRAWER.split(","))
        for ioInfo in vldList:
            if ioInfo in keys:
                self.valOutput(keys, 'F', vals, str(
                    MSGCATALOG.VIOS_IO_SELECTION_INVALID), True)
                return False
        return True

    # IO Adapters validation
    # Minimum adapter selection should be 2 per VIOS
    def validateIO(self, iolist, viosName, vios_count):
        print("\n:::: VALIDATING IO ADAPTERS ::::")
        log.info("Inside validateIO method")
        sas_adapter = CONF.get(
            'vios_profile', 'IO_ADAPTERS_STRING_SAS').split(",")
        count = 0
        total_sas_adapter = 0
        selectedAdapters = []
        isValidSelection = True
        self.only_sriov = False
        for info in iolist:
            for keys, vals in info.items():
                if vals == "Required":
                    res = self.validateNonRequiredAdapters(keys, vals)
                    if res:
                        self.valOutput(keys, 'P', vals, "", True)
                        selectedAdapters.append(keys)
                        count += 1
                    else:
                        isValidSelection = False
                else:
                    self.valOutput(keys, '-', '-', "", True)

                # counting total number of SAS adapters
                # available in excel
                for sas_info in sas_adapter:
                    if sas_info in keys:
                        total_sas_adapter = total_sas_adapter+1
        if not isValidSelection:
            return isValidSelection
        if count < 2:
            self.valOutput("IO ADAPTERS ERROR", 'F', "", str(
                MSGCATALOG.VIOS_IO_SELECTION_COUNT_INVALID), True)
            return False
        else:
            return selectedAdapters

    def filterSRIOVadapters(self, viosValidationList):
        log.info("Inside vldSRIOVadapters method")
        for data in viosValidationList:
            data['sriov_io'] = []
            data['other_io'] = []
            for IO in data['selected_io']:
                SRIOV = CONF.get('vios_profile', 'SRIOV_KEYWORD')
                if SRIOV in IO:
                    data['sriov_io'].append(IO)
                else:
                    data['other_io'].append(IO)
        return True

    def validateIOSelection(self, viosValidationList):
        log.info(" Inside validateIOSelection method")
        existingIoList = []
        isValid = True
        for data in viosValidationList:
            for io in data['selected_io']:
                if io in existingIoList:
                    self.valOutput("IO ADAPTER SELECTION ERROR", 'F', "", str(
                        MSGCATALOG.VIOS_SAME_ADAPTER_SELECTION.format(io)))
                    isValid = False
                else:
                    existingIoList.append(io)
        if isValid:
            self.filterSRIOVadapters(viosValidationList)
        return isValid

    # Validate VIOS info main function
    # Modularized as a separate function for each type of validation
    def validateViosInfo(self, info, viosHeaderName):
        log.info("Inside validateViosInfo method")
        code = self.vldEmptyFieldsVIOSname(info['NAME'], viosHeaderName)
        if code > 0:
            return code
        print("\n::::::::::::::::::::::::::::::::::::")
        print("VALIDATING : "+info['NAME'])
        print("::::::::::::::::::::::::::::::::::::")
        time.sleep(1)
        print("\n:::: VALIDATING REQUIRED FIELD ::::")
        code = self.vldEmptyFields(info)
        time.sleep(1)
        if code > 0:
            return code
        print("\n:::: VALIDATING RESOURCES ::::")
        code = self.vldNumericVals(info)
        if code == 0:
            codeRAM = self.vldRAM(
                info['MIN MEMORY(GB)'], info['DESIRED MEMORY(GB)'],
                info['MAX MEMORY(GB)'])
            codeProc = self.vldVirtualProc(
                info['MIN VIRTUAL PROCS'], info['DESIRED VIRTUAL PROCS'],
                info['MAX VIRTUAL PROCS'])
            codeProcUnit = self.vldSharedProcUnits(
                info['MIN SHARED PROC UNITS'],
                info['DESIRED SHARED PROC UNITS'],
                info['MAX SHARED PROC UNITS'])
            code = code + codeRAM + codeProc + codeProcUnit
        codeVLAN = self.vldVLAN_ID(info['PORT VLAN ID'])
        codeNW = self.vldNetworkAttributes(info)
        code = code + codeVLAN + codeNW
        return code
