# =====================================================================
# (C) 2019 Tech Mahindra Limited. All rights reserved
# No part of this code may be copied, reproduced, sold,
# or transmitted in any form or by any means, electronic,
# mechanical, photocopying, recording, or otherwise,
# without prior written permission of Tech Mahindra Limited.
#
# IN NO EVENT SHALL TECH MAHINDRA BE LIABLE TO ANY PARTY FOR DIRECT,
# INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
# INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE
# AND ITS DOCUMENTATION, EVEN IF TECH MAHINDRA HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# TECH MAHINDRA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
# BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND ACCOMPANYING
# DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS IS".
# TECH MAHINDRA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
# UPDATES, ENHANCEMENTS OR MODIFICATIONS.
# =====================================================================

from novalinkAutoInstaller.src.Common import messageCatalog as MSGCATALOG
from novalinkAutoInstaller.src.Common import autoLogger
import ipaddress
log = autoLogger.AutoLogger("InputValidatorModule")


# Contains all validation function used for VIOS and
# Novalink parameters validation
class InputValidator:

    def valOutput(self, keyName, type, val="", msg="", isIO=False):
        if type == "F":
            info = "FAILED"
        elif type == "P":
            info = "OK"
        elif type == "-":
            info = "-"
        if msg:
            msg = ": "+msg
        if isIO:
            print("%-100s %-10s %-5s %-10s" % (keyName, val, info, msg))
        else:
            print("%-40s %-50s %-5s %-10s" % (keyName, val, info, msg))

    # For validating empty keys in xlsx
    def emptyKeys(self, keyName):
        if keyName is None or keyName == "" or keyName is False:
            print("Parameters field found as blank.")
            return False
        else:
            return True

    # For validating empty values in xlsx
    def emptyValues(self, vals, infokey):
        if vals is None or vals == "" or vals is False:
            if '|' not in infokey:
                self.valOutput(
                    infokey, 'F', "", MSGCATALOG.XLSX_EMPTY_FIELDS)
                return 1
            else:
                return 0
        else:
            if '|' not in infokey:
                self.valOutput(infokey, 'P', vals)
            return 0

    # For validating number vals
    def isDigitVals(self, vals, attrName):
        if str(vals).isdigit():
            self.valOutput(attrName, 'P', vals)
            return 0
        else:
            self.valOutput(attrName, 'F', vals, MSGCATALOG.INT_VAL_INVALID)
            return 1

    # For validating floating number
    def isFloatVals(self, vals, attrName):
        try:
            infokey = float(vals)
            self.valOutput(attrName, 'P', vals)
            return 0
        except Exception as e:
            self.valOutput(attrName, 'F', vals, MSGCATALOG.DECIMAL_VAL_INVALID)
            return 1

    # Validating min, desired and max RAM
    # Validating minimum required value of RAM setup in config file
    def ramVld(self, minLimit, min, des, max):
        code = 0
        if float(min) < float(minLimit):
            self.valOutput('MIN MEMORY(GB)', 'F', str(
                min), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'MIN MEMORY', minLimit))
            code = 1
        if float(des) < float(min):
            self.valOutput('DESIRED MEMORY(GB)', 'F', str(
                des), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'DESIRED MEMORY', 'MIN MEMORY'))
            code = 1
        if float(max) < float(des):
            self.valOutput('MAX MEMORY(GB)', 'F', str(
                max), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'MAX MEMORY', 'DESIRED MEMORY'))
            code = 1
        return code

    # Validating min, desired and max Virtual Processor
    # Validating min required value of Processor setup in config file
    def vProcVld(self, minLimit, min, des, max):
        code = 0
        if float(min) < float(minLimit):
            self.valOutput('MIN VIRTUAL PROCS', 'F', str(
                min), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'MIN VIRTUAL PROCS', minLimit))
            code = 1
        if float(des) < float(min):
            self.valOutput('DESIRED VIRTUAL PROCS', 'F', str(
                des), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'DESIRED VIRTUAL PROC', 'MIN VIRTUAL PROC'))
            code = 1
        if float(max) < float(des):
            self.valOutput('MAX VIRTUAL PROCS', 'F', str(
                max), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'MAX VIRTUAL PROC', 'DESIRED VIRTUAL PROC'))
            code = 1
        return code

    # Validating min, desired and max Shared Processor Units
    # Validating min required value of Shared Processor Units
    # setup in config file
    def sharedProcVld(self, minLimit, min, des, max):
        code = 0
        if float(min) < float(minLimit):
            self.valOutput('MIN SHARED PROC UNITS', 'F', str(
                 min), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'MIN SHARED PROC UNITS', minLimit))
            code = 1
        if float(des) < float(min):
            self.valOutput('DESIRED SHARED PROC UNITS', 'F', str(
                des), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'DESIRED SHARED PROC UNITS', 'MIN SHARED PROC UNITS'))
            code = 1
        if float(max) < float(des):
            self.valOutput('MAX SHARED PROC UNITS', 'F', str(
                max), MSGCATALOG.INPUTS_LESS_THAN_REQUIRED.format(
                'MAX SHARED PROC UNITS', 'DESIRED SHARED PROC UNITS'))
            code = 1
        return code

    # Validating PORT VLAN ID
    # Range between 0 to 4094 (max Range)
    def vlanVld(self, vlanId, maxRange):
        if not str(vlanId).isdigit():
            self.valOutput(
                "PORT VLAN ID", 'F', vlanId, MSGCATALOG.VLAN_ID_INVALID)
            return 1
        elif int(vlanId) > int(maxRange) or int(vlanId) < 0:
            self.valOutput(
                "PORT VLAN ID", 'F', vlanId, MSGCATALOG.VLAN_ID_RANGE_INVALID)
            return 1
        self.valOutput("PORT VLAN ID", 'P', vlanId)
        return 0

    # Validating IP Address
    def ipAddrVld(self, ipAdd):
        try:
            ipaddress.ip_address(ipAdd)
            return 0
        except Exception as ValueError:
            return 1

    def ipAddPingVld(self, attName, ipAdd, pingable=False):
        res = self.hmcObj.pingTest(str(ipAdd))
        if pingable:
            if res['code'] == 0:
                self.valOutput(attName, 'P', ipAdd)
                return 0
            else:
                self.valOutput(attName, 'F', ipAdd, str(
                    MSGCATALOG.IP_ADD_PING_ERROR))
                return 1
        else:
            if res['code'] == 1:
                self.valOutput(attName, 'P', ipAdd)
                return 0
            elif res['code'] == 2:
                self.valOutput(attName, 'F', ipAdd, str(
                    MSGCATALOG.IP_ADDRESS_INVALID))
                return 1

            else:
                self.valOutput(attName, 'F', ipAdd, str(
                    MSGCATALOG.IP_CONFLICT_ERROR))
                return 1
